uniform sampler2D shadowcolor0;
uniform sampler2DShadow shadowtex0HW, shadowtex1HW;

#include "/lib/brdf.glsl"
#include "/lib/shadow_bias.glsl"

#if SHADOW_BLUR == 2
	f16vec3 sample_sm(float16_t mul, vec3 s_scrn, ivec2 offset) {
		immut float16_t solid_vis = float16_t(textureLodOffset(shadowtex1HW, s_scrn, 0.0, offset));

		if (solid_vis > float16_t(0.0)) {
			immut float16_t trans_vis = float16_t(textureLodOffset(shadowtex0HW, s_scrn, 0.0, offset));

			f16vec3 color = (mul * solid_vis).xxx;

			if (trans_vis < solid_vis) {
				color = mix(color * f16vec3(textureLodOffset(shadowcolor0, s_scrn.xy, 0.0, offset).rgb), color, trans_vis);
			}

			return color;
		} else return f16vec3(0.0);
	}
#endif

f16vec3 sample_shadow(vec3 s_scrn) {
	#if SHADOW_BLUR < 2
		#if !SHADOW_BLUR
			/*
				const bool shadowtex0Nearest = true;
				const bool shadowtex1Nearest = true;
				const bool shadowcolor0Nearest = true;
			*/
		#endif

		immut float16_t solid_vis = float16_t(textureLod(shadowtex1HW, s_scrn, 0.0));

		if (solid_vis > float16_t(0.0)) {
			immut float16_t trans_vis = float16_t(textureLod(shadowtex0HW, s_scrn, 0.0));

			f16vec3 color = solid_vis.xxx;

			if (trans_vis < solid_vis) {
				color = mix(color * f16vec3(textureLod(shadowcolor0, s_scrn.xy, 0.0).rgb), color, trans_vis);
			}

			return color;
		} else return f16vec3(0.0);
	#else
		// Gaussean blur approximation based on: https://web.archive.org/web/20230210095515/http://the-witness.net/news/2013/09/shadow-mapping-summary-part-1/

		const float sm_res = float(shadowMapResolution);

		vec2 base_uv;
		immut f16vec2 st = f16vec2(modf(fma(s_scrn.xy, sm_res.xx, vec2(0.5)), base_uv));
		base_uv = fma(base_uv, vec2(1.0 / sm_res), vec2(-0.5 / sm_res));

		immut f16vec2 uvw0 = fma(st, f16vec2(-3.0), f16vec2(4.0));
		const vec2 uvw1_32 = vec2(7.0); // might as well do const 32-bit if we can
		const f16vec2 uvw1 = f16vec2(uvw1_32);
		immut f16vec2 uvw2 = fma(st, f16vec2(3.0), f16vec2(1.0));

		immut vec2 uv0 = vec2(fma(st, f16vec2(-2.0 / sm_res), f16vec2(3.0 / sm_res)) / uvw0);
		immut vec2 uv1 = vec2(fma(st, f16vec2(1.0 / (sm_res * uvw1_32)), f16vec2(3.0 / (sm_res * uvw1_32))));
		immut vec2 uv2 = vec2(st / (float16_t(sm_res) * uvw2));

		return (
			uvw0.y * (
				sample_sm(uvw0.x, vec3(base_uv + vec2(uv0.x, uv0.y), s_scrn.z), ivec2(-2, -2)) +
				sample_sm(uvw1.x, vec3(base_uv + vec2(uv1.x, uv0.y), s_scrn.z), ivec2(0, -2)) +
				sample_sm(uvw2.x, vec3(base_uv + vec2(uv2.x, uv0.y), s_scrn.z), ivec2(2, -2))
			) + uvw1.y * (
				sample_sm(uvw0.x, vec3(base_uv + vec2(uv0.x, uv1.y), s_scrn.z), ivec2(-2, 0)) +
				sample_sm(uvw1.x, vec3(base_uv + vec2(uv1.x, uv1.y), s_scrn.z), ivec2(0, 0)) +
				sample_sm(uvw2.x, vec3(base_uv + vec2(uv2.x, uv1.y), s_scrn.z), ivec2(2, 0))
			) + uvw2.y * (
				sample_sm(uvw0.x, vec3(base_uv + vec2(uv0.x, uv2.y), s_scrn.z), ivec2(-2, 2)) +
				sample_sm(uvw1.x, vec3(base_uv + vec2(uv1.x, uv2.y), s_scrn.z), ivec2(0, 2)) +
				sample_sm(uvw2.x, vec3(base_uv + vec2(uv2.x, uv2.y), s_scrn.z), ivec2(2, 2))
			)
		) / float16_t(144.0);
	#endif
}